package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Property;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.DispositivoEmbarcadaDAO;
import com.rjconsultores.ventaboletos.entidad.DispositivoEmbarcada;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FormaPagoDet;
import com.rjconsultores.ventaboletos.entidad.PtoVtaSeguro;
import com.rjconsultores.ventaboletos.entidad.PtovtaCatInd;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;

@SuppressWarnings("unchecked")
@Repository("dispositivoEmbarcadaDAO")
public class DispositivoEmbarcadaHibernateDAO extends GenericHibernateDAO<DispositivoEmbarcada, Long>
		implements DispositivoEmbarcadaDAO {

	@Autowired
	public DispositivoEmbarcadaHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public List<DispositivoEmbarcada> obtenerTodos() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		c.addOrder(Order.asc("descricao"));

		return c.list();
	}

	@Override
	public DispositivoEmbarcada buscarVersaoUltimaSinc(String versaoUltimaSinc) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		c.add(Restrictions.eq("versaoUltimaSinc", versaoUltimaSinc));

		return (DispositivoEmbarcada) c.uniqueResult();
	}

	@Override
	public List<String> buscarAllVersaoUltimaSinc() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.setProjection(Projections.distinct(Property.forName("versaoUltimaSinc")));		
		c.add(Restrictions.isNotNull("versaoUltimaSinc"));
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		c.addOrder(Order.asc("versaoUltimaSinc"));
		
		return (List<String>) c.list();
	}

	@Override
	public DispositivoEmbarcada buscarImeiDuplicado(String imei){
		StringBuilder sb = new StringBuilder();
		sb.append("select dp ");
		sb.append("from DispositivoEmbarcada dp ");
		sb.append("where dp.imei = :imei ");
		
		Query query = getSession().createQuery(sb.toString());
		query.setString("imei", imei);
		
		List<DispositivoEmbarcada> list = query.list();
		
		if (list.isEmpty()) {
			return null;
		}
		
		return (DispositivoEmbarcada) list.get(0);

	}
	
	@Override
	public DispositivoEmbarcada buscarImei(String imei) {
		StringBuilder sb = new StringBuilder();
		sb.append("select dp ");
		sb.append("from DispositivoEmbarcada dp ");
		sb.append(" inner join fetch dp.series as se ");
		sb.append("where se.activo = 1 ");
		sb.append(" and se.indBaixado = 0 ");
		sb.append("and dp.imei = :imei ");
		sb.append("and dp.activo = 1 ");
		Query query = getSession().createQuery(sb.toString());
		query.setString("imei", imei);
		return (DispositivoEmbarcada) query.uniqueResult();

	}

	@Override
	@Transactional
	public DispositivoEmbarcada suscribir(DispositivoEmbarcada entity) {
		try {
			this.getHibernateTemplate().save(entity);
			getHibernateTemplate().flush();

			return entity;
		} catch (final HibernateException ex) {

			throw convertHibernateAccessException(ex);
		}
	}

	@Override
	@Transactional
	public DispositivoEmbarcada actualizacion(DispositivoEmbarcada entity) {
		entity = getHibernateTemplate().merge(entity);
		getHibernateTemplate().flush();
		return entity;
	}

	@Override
	@Transactional
	public DispositivoEmbarcada suscribirActualizar(DispositivoEmbarcada dispositivoEmbarcada) {
		if (buscarDispositivoPorId(dispositivoEmbarcada) != null) {
			actualizacion(dispositivoEmbarcada);
		} else {
			suscribir(dispositivoEmbarcada);
		}
		return dispositivoEmbarcada;
	}

	@Override
	public DispositivoEmbarcada buscarDispositivoPorId(DispositivoEmbarcada dispositivoEmbarcada) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

		c.add(Restrictions.eq("id", dispositivoEmbarcada.getDispositivoEmbarcadaId()));

		return (DispositivoEmbarcada) c.uniqueResult();
	}

	@Override
	public List<DispositivoEmbarcada> buscarPorEmpresaPuntoVenta(Empresa empresa, PuntoVenta puntoventa) {
		StringBuilder sb = new StringBuilder();
		sb.append("from DispositivoEmbarcada de ");
		sb.append("where activo = 1 ");
		sb.append("and de.empresaId = :empresaId ");
		sb.append("and de.puntoVentaId = :puntoVentaId ");

		Query qry = getSession().createQuery(sb.toString());
		qry.setLong("empresaId", empresa.getEmpresaId());
		qry.setLong("puntoVentaId", puntoventa.getPuntoventaId());

		return qry.list();
	}

	@Override
	public List<DispositivoEmbarcada> buscarTodos() {
		StringBuilder sb = new StringBuilder();
		sb.append("from DispositivoEmbarcada de ");
		sb.append("where activo = 1 ");

		Query qry = getSession().createQuery(sb.toString());

		return qry.list();
	}
	
	@Override
	public List<FormaPagoDet> getListaFormaPago(Integer puntoventaId) {
		String queryStr = "from FormaPagoDet fp where fp.activo = true and fp.indEmbarcada = true and fp.puntoVenta.puntoventaId = :puntoventaId";

		Query query = getSession().createQuery(queryStr);
		query.setInteger("puntoventaId", puntoventaId);

		return query.list();
	}

	@Override
	public List<PtoVtaSeguro> getListaSeguroTaxaEmbarque(Integer puntoventaId) {
		String queryStr = "from PtoVtaSeguro x where x.activo = true and x.puntoventaId.puntoventaId = :puntoventaId";

		Query query = getSession().createQuery(queryStr);
		query.setInteger("puntoventaId", puntoventaId);

		return query.list();
	}

	@Override
	public List<PtovtaCatInd> getListaTiposPassagensBloqueadas(Integer puntoventaId) {
		String queryStr = "from PtovtaCatInd x where x.activo = true and x.indEmbarcada = true and x.puntoVenta.puntoventaId = :puntoventaId";

		Query query = getSession().createQuery(queryStr);
		query.setInteger("puntoventaId", puntoventaId);

		return query.list();
	}

}
