package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.Hibernate;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

@AuditarClasse(nome = "HistoricoPuntoVenta", tela = "auditarClasse.HistoricoPuntoVenta")
@Entity
@SequenceGenerator(name = "HIST_PUNTOVENTA_SEQ", sequenceName = "HIST_PUNTOVENTA_SEQ", allocationSize = 1)
@Table(name = "HIST_PUNTOVENTA")
public class HistoricoPuntoVenta implements Serializable, Auditavel<HistoricoPuntoVenta> {

	private static final long serialVersionUID = 1L;
	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "HIST_PUNTOVENTA_SEQ")
	@Column(name = "HIST_PUNTOVENTA_ID")
	private Integer historicoPuntoVentaId;

	@Column(name = "IND_BLOQUEIO")
	private Boolean indBloqueio;

	@Column(name = "FEC_BLOQUEIO")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fecBloqueio;

	@NaoAuditar
	@ManyToOne
	@JoinColumn(name = "PUNTOVENTA_ID")
	private PuntoVenta puntoVenta;

	@Column(name = "ACTIVO")
	private Boolean activo;

	@Column(name = "FECMODIF")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fecmodif;

	@Column(name = "USUARIO_ID")
	private Integer usuarioId;
	
	
	@Column(name = "MOTIVO_BLOQUEIO")
	private String motivoBloqueio;

	@Transient
	@NaoAuditar
	private HistoricoPuntoVenta historicoPuntoVentaClone;
	
	public Integer getHistoricoPuntoVentaId() {
		return historicoPuntoVentaId;
	}

	public void setHistoricoPuntoVentaId(Integer historicoPuntoVentaId) {
		this.historicoPuntoVentaId = historicoPuntoVentaId;
	}

	public Boolean getIndBloqueio() {
		return indBloqueio;
	}

	public void setIndBloqueio(Boolean indBloqueio) {
		this.indBloqueio = indBloqueio;
	}

	public Date getFecBloqueio() {
		return fecBloqueio;
	}

	public void setFecBloqueio(Date fecBloqueio) {
		this.fecBloqueio = fecBloqueio;
	}

	public PuntoVenta getPuntoVenta() {
		return puntoVenta;
	}

	public void setPuntoVenta(PuntoVenta puntoVenta) {
		this.puntoVenta = puntoVenta;
	}

	public Boolean getActivo() {
		return activo;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public Integer getUsuarioId() {
		return usuarioId;
	}

	public void setUsuarioId(Integer usuarioId) {
		this.usuarioId = usuarioId;
	}

	@Override
	public void clonar() throws CloneNotSupportedException {
		historicoPuntoVentaClone = new HistoricoPuntoVenta();
		historicoPuntoVentaClone = (HistoricoPuntoVenta) this.clone();
		Hibernate.initialize(historicoPuntoVentaClone.getPuntoVenta());
		
	}

	@Override
	public HistoricoPuntoVenta getCloneObject() throws CloneNotSupportedException {
		return historicoPuntoVentaClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getHistoricoPuntoVentaId());
	}

	public String getMotivoBloqueio() {
		return motivoBloqueio;
	}

	public void setMotivoBloqueio(String motivoBloqueio) {
		this.motivoBloqueio = motivoBloqueio;
	}
	
	
	
	
	
	
}
