package com.rjconsultores.ventaboletos.service.impl;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CobrancaAdcPuntoVentaDAO;
import com.rjconsultores.ventaboletos.entidad.CobrancaAdcPuntoVenta;
import com.rjconsultores.ventaboletos.entidad.CobrancaAdicionalHistorico;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.service.CobrancaAdcPuntoVentaService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("cobrancaAdcPuntoVentaService")
public class CobrancaAdcPuntoVentaServiceImpl implements CobrancaAdcPuntoVentaService {

	@Autowired
	CobrancaAdcPuntoVentaDAO cobrancaAdcPuntoVentaDAO;
	
	@Override
	public List<CobrancaAdcPuntoVenta> obtenerTodos() {
		return cobrancaAdcPuntoVentaDAO.obtenerTodos();		
	}

	@Override
	public CobrancaAdcPuntoVenta obtenerID(Integer id) {
		return cobrancaAdcPuntoVentaDAO.obtenerID(id);
	}

	@Override
	@Transactional
	public CobrancaAdcPuntoVenta suscribir(CobrancaAdcPuntoVenta entidad) {
		entidad.setActivo(true);
		entidad.setFecmodif(new Date());
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		return cobrancaAdcPuntoVentaDAO.suscribir(entidad);
	}

	@Override
	@Transactional
	public CobrancaAdcPuntoVenta actualizacion(CobrancaAdcPuntoVenta entidad) {
		entidad.setFecmodif(new Date());
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		return cobrancaAdcPuntoVentaDAO.actualizacion(entidad);
	}

	@Override
	@Transactional
	public void borrar(CobrancaAdcPuntoVenta entidad) {
		entidad.setActivo(false);
		entidad.setFecmodif(new Date());
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		cobrancaAdcPuntoVentaDAO.actualizacion(entidad);
	}
	
	public List<CobrancaAdicionalHistorico> buscaHistoricoCobranca(CobrancaAdcPuntoVenta cobranca, Date dataInicial, Date dataFinal) {
		return cobrancaAdcPuntoVentaDAO.buscaHistoricoCobranca(cobranca, dataInicial, dataFinal);
	}

	@Override
	public List<CobrancaAdcPuntoVenta> buscarTodasAsCobrancas(PuntoVenta puntoVenta) {
		return cobrancaAdcPuntoVentaDAO.buscarTodasAsCobrancas(puntoVenta);
	}

}
