package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.DescontoComissaoDAO;
import com.rjconsultores.ventaboletos.entidad.DescontoComissao;
import com.rjconsultores.ventaboletos.service.DescontoComissaoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("descontoComissaoService")
public class DescontoComissaoServiceImpl implements DescontoComissaoService {

	@Autowired
	private DescontoComissaoDAO descontoComissaoDAO;

	@Override
	public List<DescontoComissao> obtenerTodos() {
		return descontoComissaoDAO.obtenerTodos();
	}

	@Override
	public DescontoComissao obtenerID(Long id) {
		return descontoComissaoDAO.obtenerID(id);
	}

	@Override
	public List<DescontoComissao> buscaDescontoComissaoPeriodo(Integer puntoVentaId, Integer empresaId, Date inicioPeriodo, Date fimPeriodo) {
		return descontoComissaoDAO.buscaDescontoComissao(puntoVentaId, empresaId, inicioPeriodo, fimPeriodo);
	}

	@Transactional
	@Override
	public DescontoComissao suscribir(DescontoComissao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return descontoComissaoDAO.suscribir(entidad);
	}

	@Transactional
	@Override
	public DescontoComissao actualizacion(DescontoComissao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return descontoComissaoDAO.actualizacion(entidad);
	}

	@Transactional
	@Override
	public void borrar(DescontoComissao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		descontoComissaoDAO.actualizacion(entidad);
	}

	@Override
	@Transactional
	public DescontoComissao suscribirOrActualizacion(DescontoComissao entidad) {
		return entidad != null && entidad.getDescontocomissaoId() == null ? suscribir(entidad) : actualizacion(entidad);
	}

}
