package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.OrgaoComprovanteCustomizadoDAO;
import com.rjconsultores.ventaboletos.entidad.OrgaoComprovanteCustomizado;
import com.rjconsultores.ventaboletos.entidad.OrgaoConcedente;
import com.rjconsultores.ventaboletos.service.OrgaoComprovanteCustomizadoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("orgaoComprovanteCustomizadoService")
public class OrgaoComprovanteCustomizadoServiceImpl implements OrgaoComprovanteCustomizadoService {

	@Autowired
	private OrgaoComprovanteCustomizadoDAO orgaoComprovanteCustomizadoServiceDAO;

	@Override
	public List<OrgaoComprovanteCustomizado> obtenerTodos() {
		return orgaoComprovanteCustomizadoServiceDAO.obtenerTodos();
	}

	@Override
	public OrgaoComprovanteCustomizado obtenerID(Integer id) {
		return orgaoComprovanteCustomizadoServiceDAO.obtenerID(id);
	}

	@Override
	@Transactional
	public OrgaoComprovanteCustomizado suscribir(OrgaoComprovanteCustomizado entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return orgaoComprovanteCustomizadoServiceDAO.suscribir(entidad);
	}

	@Override
	@Transactional
	public OrgaoComprovanteCustomizado actualizacion(OrgaoComprovanteCustomizado entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
	

		return orgaoComprovanteCustomizadoServiceDAO.actualizacion(entidad);
	}

	@Override
	@Transactional
	public void borrar(OrgaoComprovanteCustomizado entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		orgaoComprovanteCustomizadoServiceDAO.actualizacion(entidad);

	}

	@Override
	public OrgaoComprovanteCustomizado buscarPorOrgao(OrgaoConcedente orgaoConcendente, String cveTipoComprovante) {
		return orgaoComprovanteCustomizadoServiceDAO.buscarPorOrgao(orgaoConcendente, cveTipoComprovante);
	}
	
	
	@Override
	public OrgaoComprovanteCustomizado buscarPorOrgaoPorTipoComprovanteId(OrgaoConcedente orgaoConcedente, Integer tipoComprovanteId) {
		return orgaoComprovanteCustomizadoServiceDAO.buscarPorOrgaoPorTipoComprovanteId(orgaoConcedente, tipoComprovanteId);
	}

	@Override
	public List<OrgaoComprovanteCustomizado> buscarPorOrgao(OrgaoConcedente OrgaoConcedente) {
		// TODO Auto-generated method stub
		return orgaoComprovanteCustomizadoServiceDAO.buscarOrgaoPorOgao(OrgaoConcedente);
	}

}
